/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * sound.c - Sound data input functions
 * 
 */



//*** Includes

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <errno.h>

#include <sys/soundcard.h>

#include "main.h"



/*
 * sound_init
 *
 * Return 0 on success
 *
 */

int sound_init (int desired_samplerate)
{
int int1;
int format;
int channels;

// Open sound device
sound_fd = open (gstr_sounddev->str, O_RDONLY);
if (sound_fd <= 0)
    {
    PRINTF_DEBUG("Error opening %s: %s\n", gstr_sounddev->str, strerror(errno));
    return (-1);
    }

// Set format
format = AFMT_U8;
int1 = ioctl(sound_fd, SNDCTL_DSP_SETFMT, &format);
if (int1 != 0)
    {
    PRINTF_DEBUG("Could not set audio format\n");
    close(sound_fd);
    return (-1);
    }

// Set number of channels
channels = 1;
int1 = ioctl(sound_fd, SNDCTL_DSP_CHANNELS, &channels);
if (int1 != 0)
    {
    PRINTF_DEBUG("Could not set number of channels\n");
    close(sound_fd);
    return (-1);
    }

// Set samplerate
samplerate = desired_samplerate;
int1 = ioctl(sound_fd, SNDCTL_DSP_SPEED, &samplerate);
if (int1 != 0)
    {
    PRINTF_DEBUG("Could not set samplerate\n");
    close(sound_fd);
    return (-1);
    }

// Check if the parameters were set correctly
if (format != AFMT_U8)
    {
    PRINTF_DEBUG("Desired audio format not supported\n");
    close(sound_fd);
    return (-1);
    }

if (channels != 1)
    {
    PRINTF_DEBUG("Could not set recording to mono\n");
    close(sound_fd);
    return (-1);
    }

if ((samplerate > 1000000) || (samplerate < 1000))
    {
    PRINTF_DEBUG("Samplerate is %d. Can't accept that.\n", samplerate);
    close(sound_fd);
    return (-1);
    }

int1=fcntl(sound_fd, F_SETFL, O_NONBLOCK);
if (int1 == -1)
    {
    PRINTF_DEBUG("Error setting %s to nonblocking mode\n", gstr_sounddev->str);
    close(sound_fd);
    return (-1);
    }

return(0);
}



/*
 * sound_deinit
 *
 */

void sound_deinit()
{
int int1;

if (sound_fd >= 0)
    {
    int1=close(sound_fd);
    if (int1 != 0)
        PRINTF_DEBUG("Error closing sound device\n");
    sound_fd=-1;
    }

return;
}



/*
 * sound_readtobuffer
 *
 * Returns 0 on success
 *
 */

int sound_readtobuffer ()
{
int int1;
int int2;
int int3;
int writepos_0;
int return_value=0;
u_int8_t chars[8];


int2 = soundbuffer_len;

soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
if (soundbuffer_writepos < soundbuffer_readpos)
    {
    soundbuffer_writepos += SOUNDBUFFER_SIZE;
    }

writepos_0=soundbuffer_writepos;

label0:
int1 = read (sound_fd, &(chars[0]), 8); // Q: Why read 8 bytes? A: Seems some
                                        // systems have trouble feeding us
                                        // bytes one by one. (Could be a
                                        // format / channels problem.)
if ((int1 >= 1) && (int1 <= 8))
    {
    // Got some data
    for (int3 = 0; int3 < int1; int3++)
        soundbuffer[soundbuffer_writepos++ % SOUNDBUFFER_SIZE] = chars[int3];
    
    goto label0;
    }
else if (int1 == 0)
    {
    // Nothing read
    }
else
    {
    if (!(errno == EAGAIN)) // Nonblocking is not an error
        {
        PRINTF_DEBUG("Error reading from sound device (Debug info:%d, %s)\n",
	    int1, strerror(errno));
        return_value=1;
        }
    }

// Some samples in the buffer might have been discarded,
// so update soundbuffer_readpos
if ( (soundbuffer_writepos - soundbuffer_readpos) >= SOUNDBUFFER_SIZE )
    {
    soundbuffer_readpos = soundbuffer_writepos + 1;
    }

// Update backto_lastdraw_samplecount
backto_lastdraw_samplecount-=(soundbuffer_writepos - writepos_0);

// Update soundbuffer_len
soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
if (soundbuffer_writepos < soundbuffer_readpos)
    {
    soundbuffer_writepos += SOUNDBUFFER_SIZE;
    }
soundbuffer_len = soundbuffer_writepos - soundbuffer_readpos;

if (soundbuffer_len < int2) PRINTF_DEBUG("soundbuffer_len < int2");

return(0);
}


